@@ -349,6 +349,13 @@ struct p_protocol {
 #define UUID_FLAG_CRASHED_PRIMARY 2
 #define UUID_FLAG_INCONSISTENT 4
 #define UUID_FLAG_SKIP_INITIAL_SYNC 8
+
+#define UUID_FLAG_MASK_COMPAT_84 \
+	(UUID_FLAG_DISCARD_MY_DATA|\
+	 UUID_FLAG_CRASHED_PRIMARY|\
+	 UUID_FLAG_INCONSISTENT|\
+	 UUID_FLAG_SKIP_INITIAL_SYNC)
+
 #define UUID_FLAG_NEW_DATAGEN 16
 #define UUID_FLAG_STABLE 32
 #define UUID_FLAG_GOT_STABLE 64 /* send UUIDs */
