@@ -20,6 +20,13 @@
 #include <Configurable.h>
 #include <Configurator.h>
 #include <comparators.h>
+#include <IntervalTimer.h>
+#include <colormodes.h>
+
+extern "C"
+{
+    #include <time.h>
+}
 
 class DrbdMon : public Configurable, public Configurator
 {
@@ -29,10 +36,24 @@ class DrbdMon : public Configurable, public Configurator
 
     static const std::string TOKEN_DELIMITER;
 
+    static const char DEBUG_SEQ_PFX;
+
     static const std::string OPT_HELP_KEY;
     static const std::string OPT_VERSION_KEY;
+    static const std::string OPT_FREQ_LMT_KEY;
     static const ConfigOption OPT_HELP;
     static const ConfigOption OPT_VERSION;
+    static const ConfigOption OPT_FREQ_LMT;
+
+    // Environment variable for color mode selection
+    static const char* ENV_COLOR_MODE;
+    static const char* COLOR_MODE_EXTENDED;
+    static const char* COLOR_MODE_BASIC;
+
+    static const std::string UNIT_SFX_SECONDS;
+    static const std::string UNIT_SFX_MILLISECONDS;
+    static const uint16_t MAX_INTERVAL;
+    static const uint16_t DFLT_INTERVAL;
 
     static const std::string TYPE_RESOURCE;
     static const std::string TYPE_CONNECTION;
@@ -47,7 +68,6 @@ class DrbdMon : public Configurable, public Configurator
 
     static const char HOTKEY_QUIT;
     static const char HOTKEY_REPAINT;
-    static const char HOTKEY_CLEAR_MSG;
     static const char HOTKEY_REINIT;
     static const char HOTKEY_VERSION;
 
@@ -80,16 +100,21 @@ class DrbdMon : public Configurable, public Configurator
         RESTART_IMMED,
         RESTART_DELAYED,
         TERMINATE,
-        TERMINATE_NO_CLEAR
+        TERMINATE_NO_CLEAR,
+        DEBUG_MODE
     };
 
+    const color_mode drbdmon_colors;
+
     // @throws std::bad_alloc
     DrbdMon(
         int argc,
         char* argv[],
         MessageLog& log_ref,
+        MessageLog& debug_log_ref,
         fail_info& fail_data_ref,
-        const std::string* const node_name_ref
+        const std::string* const node_name_ref,
+        const color_mode colors
     );
     DrbdMon(const DrbdMon& orig) = delete;
     DrbdMon& operator=(const DrbdMon& orig) = delete;
@@ -107,7 +132,8 @@ class DrbdMon : public Configurable, public Configurator
     virtual void process_event_message(
         std::string& mode,
         std::string& type,
-        PropsMap& event_props
+        PropsMap& event_props,
+        std::string& event_line
     );
     // Returns the action requested to be taken upon return from this class' run() method.
     // This method should be called only after run() has returned.
@@ -150,6 +176,7 @@ class DrbdMon : public Configurable, public Configurator
     fail_info&    fail_data;
     finish_action fin_action {DrbdMon::finish_action::RESTART_IMMED};
     MessageLog&   log;
+    MessageLog&   debug_log;
     const std::string* const node_name;
 
     bool          shutdown      {false};
@@ -160,46 +187,51 @@ class DrbdMon : public Configurable, public Configurator
     std::unique_ptr<GenericDisplay>  display {nullptr};
     std::unique_ptr<Configurable*[]> configurables {nullptr};
 
+    std::unique_ptr<IntervalTimer> interval_timer_mgr {nullptr};
+    struct timespec prev_timestamp {0, 0};
+    struct timespec cur_timestamp {0, 0};
+    bool use_dflt_freq_lmt {true};
+
     // @throws std::bad_alloc, EventMessageException
-    void create_connection(PropsMap& event_props);
+    void create_connection(PropsMap& event_props, const std::string& event_line);
     // @throws std::bad_alloc, EventMessageException
-    void create_device(PropsMap& event_props);
+    void create_device(PropsMap& event_props, const std::string& event_line);
     // @throws std::bad_alloc, EventMessageException
-    void create_peer_device(PropsMap& event_props);
+    void create_peer_device(PropsMap& event_props, const std::string& event_line);
     // @throws std::bad_alloc, EventMessageException
-    void create_resource(PropsMap& event_props);
+    void create_resource(PropsMap& event_props, const std::string& event_line);
 
     // @throws EventMessageException, EventObjectException
-    void update_connection(PropsMap& event_props);
+    void update_connection(PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException, EventObjectException
-    void update_device(PropsMap& event_props);
+    void update_device(PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException, EventObjectException
-    void update_peer_device(PropsMap& event_props);
+    void update_peer_device(PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException, EventObjectException
-    void update_resource(PropsMap& event_props);
+    void update_resource(PropsMap& event_props, const std::string& event_line);
 
     // @throws EventMessageException
-    void destroy_connection(PropsMap& event_props);
+    void destroy_connection(PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException
-    void destroy_device(PropsMap& event_props);
+    void destroy_device(PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException
-    void destroy_peer_device(PropsMap& event_props);
+    void destroy_peer_device(PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException
-    void destroy_resource(PropsMap& event_props);
+    void destroy_resource(PropsMap& event_props, const std::string& event_line);
 
     // @throws EventMessageException, EventObjectException
-    DrbdConnection& get_connection(DrbdResource& res, PropsMap& event_props);
+    DrbdConnection& get_connection(DrbdResource& res, PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException, EventObjectException
-    DrbdVolume& get_device(VolumesContainer& vol_con, PropsMap& event_props);
+    DrbdVolume& get_device(VolumesContainer& vol_con, PropsMap& event_props, const std::string& event_line);
     // @throws EventMessageException, EventObjectException
-    DrbdResource& get_resource(PropsMap& event_props);
+    DrbdResource& get_resource(PropsMap& event_props, const std::string& event_line);
 
     // Loads the event_props map with the properties contained in tokens
     //
     // @param: tokens StringTokenizer that returns the 'key:value' tokens from a 'drbdsetup event2' line
     // @param: event_props Property map to load the key == value mappings into
     // @throws std::bad_alloc
-    void parse_event_props(StringTokenizer& tokens, PropsMap& event_props);
+    void parse_event_props(StringTokenizer& tokens, PropsMap& event_props, std::string& event_line);
     // Clears the event_props map and frees all its entries
     void clear_event_props(PropsMap& event_props);
 
@@ -217,6 +249,14 @@ class DrbdMon : public Configurable, public Configurator
 
     void problem_counter_update(StateFlags::state res_last_state, StateFlags::state res_new_state) noexcept;
 
+    void disable_interval_timer(bool& timer_available, bool& timer_armed) noexcept;
+
+    // @throws TimerException
+    inline bool is_interval_exceeded();
+
+    inline void cond_display_update(bool& timer_available, bool& timer_armed) noexcept;
+    inline void update_timestamp(bool& timer_available, bool& timer_armed) noexcept;
+
     // Frees resources
     // @throws std::bad_alloc
     void cleanup(
