@@ -30,20 +30,27 @@ const char* DrbdRole::get_role_label() const
     return label;
 }
 
-// @throws EventMessageException
+// @throws std::bad_alloc, EventMessageException
 DrbdRole::resource_role DrbdRole::parse_role(std::string& role_name)
 {
     DrbdRole::resource_role role = DrbdRole::resource_role::UNKNOWN;
 
-    if (role_name == "Primary")
+    if (role_name == ROLE_LABEL_PRIMARY)
     {
         role = DrbdRole::resource_role::PRIMARY;
     }
     else
-    if (role_name == "Secondary")
+    if (role_name == ROLE_LABEL_SECONDARY)
     {
         role = DrbdRole::resource_role::SECONDARY;
     }
+    else
+    if (role_name != ROLE_LABEL_UNKNOWN)
+    {
+        std::string error_msg("Invalid DRBD event: Invalid DRBD role");
+        std::string debug_info("Invalid DRBD role");
+        throw new EventMessageException(&error_msg, &debug_info, nullptr);
+    }
 
     return role;
 }
