@@ -1,10 +1,10 @@
 /**
  * Quick balanced binary search tree
  *
- * @version 2017-02-14_001
+ * @version 2018-07-13_001
  * @author  Robert Altnoeder (r.altnoeder@gmx.net)
  *
- * Copyright (C) 2012 - 2017 Robert ALTNOEDER
+ * Copyright (C) 2012 - 2018 Robert ALTNOEDER
  *
  * Redistribution and use in source and binary forms,
  * with or without modification, are permitted provided that
@@ -55,15 +55,15 @@ class QTree : public dsaext::Map<K, V>
         friend class QTree;
 
       private:
-        K*    key     {nullptr};
-        V*    value   {nullptr};
-        Node* less    {nullptr};
-        Node* greater {nullptr};
-        Node* parent  {nullptr};
-        int   balance {0};
+        const K*    key     {nullptr};
+        const V*    value   {nullptr};
+        Node*       less    {nullptr};
+        Node*       greater {nullptr};
+        Node*       parent  {nullptr};
+        int         balance {0};
 
       public:
-        Node(K* key_ptr, V* value_ptr)
+        Node(const K* key_ptr, const V* value_ptr)
         {
             key = key_ptr;
             value = value_ptr;
@@ -92,12 +92,83 @@ class QTree : public dsaext::Map<K, V>
 
         virtual K* get_key() const
         {
-            return key;
+            return const_cast<K*> (key);
         }
 
         virtual V* get_value() const
         {
-            return value;
+            return const_cast<V*> (value);
+        }
+
+        virtual V* set_value(const V* new_value)
+        {
+            const V* prev_value = value;
+            value = new_value;
+            return const_cast<V*> (prev_value);
+        }
+
+        virtual Node* successor() const
+        {
+            const Node* ret_node = this;
+
+            if (ret_node->greater != nullptr)
+            {
+                ret_node = ret_node->greater;
+                while (ret_node->less != nullptr)
+                {
+                    ret_node = ret_node->less;
+                }
+            }
+            else
+            {
+                do
+                {
+                    if (ret_node->parent != nullptr)
+                    {
+                        if (ret_node->parent->less == ret_node)
+                        {
+                            ret_node = ret_node->parent;
+                            break;
+                        }
+                    }
+                    ret_node = ret_node->parent;
+                }
+                while (ret_node != nullptr);
+            }
+
+            return const_cast<Node*> (ret_node);
+        }
+
+        virtual Node* predecessor() const
+        {
+            const Node* ret_node = this;
+
+            if (ret_node->less != nullptr)
+            {
+                ret_node = ret_node->less;
+                while (ret_node->greater != nullptr)
+                {
+                    ret_node = ret_node->greater;
+                }
+            }
+            else
+            {
+                do
+                {
+                    if (ret_node->parent != nullptr)
+                    {
+                        if (ret_node->parent->greater == ret_node)
+                        {
+                            ret_node = ret_node->parent;
+                            break;
+                        }
+                    }
+                    ret_node = ret_node->parent;
+                }
+                while (ret_node != nullptr);
+            }
+
+            return const_cast<Node*> (ret_node);
         }
 
         virtual void reuse()
@@ -117,15 +188,14 @@ class QTree : public dsaext::Map<K, V>
     {
       public:
         BaseIterator(QTree<K, V>& qtree_ref):
-            qtree_obj(qtree_ref)
+            qtree_obj(&qtree_ref)
         {
-            if (qtree_obj.root != nullptr)
+            if (qtree_obj->root != nullptr)
             {
-                for (iter_node = qtree_obj.root;
-                     iter_node->less != nullptr;
-                     iter_node = iter_node->less)
+                iter_node = qtree_obj->root;
+                while (iter_node->less != nullptr)
                 {
-                    // intentional no-op block
+                    iter_node = iter_node->less;
                 }
             }
             else
@@ -134,6 +204,12 @@ class QTree : public dsaext::Map<K, V>
             }
         }
 
+        BaseIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            qtree_obj(&qtree_ref),
+            iter_node(&start_node)
+        {
+        }
+
         BaseIterator(const BaseIterator& orig) = default;
         BaseIterator& operator=(const BaseIterator& orig) = default;
         BaseIterator(BaseIterator&& orig) = default;
@@ -152,7 +228,7 @@ class QTree : public dsaext::Map<K, V>
 
         virtual size_t get_size() const
         {
-            return qtree_obj.size;
+            return qtree_obj->size;
         }
 
       protected:
@@ -164,35 +240,106 @@ class QTree : public dsaext::Map<K, V>
             {
                 if (iter_node->greater != nullptr)
                 {
-                    for (iter_node = iter_node->greater;
-                         iter_node->less != nullptr;
-                         iter_node = iter_node->less)
+                    iter_node = iter_node->greater;
+                    while (iter_node->less != nullptr)
                     {
-                        // intentional no-op block
+                        iter_node = iter_node->less;
                     }
                 }
                 else
                 {
-                    do
+                    while (iter_node->parent != nullptr && iter_node->parent->greater == iter_node)
                     {
-                        if (iter_node->parent != nullptr)
-                        {
-                            if (iter_node->parent->less == iter_node)
-                            {
-                                iter_node = iter_node->parent;
-                                break;
-                            }
-                        }
                         iter_node = iter_node->parent;
                     }
-                    while (iter_node != nullptr);
+                    iter_node = iter_node->parent;
                 }
             }
 
             return ret_node;
         }
       private:
-        QTree& qtree_obj;
+        const QTree* qtree_obj;
+        Node*  iter_node {nullptr};
+    };
+
+    template<typename T>
+    class BaseReverseIterator : public dsaext::QIterator<T>
+    {
+      public:
+        BaseReverseIterator(QTree<K, V>& qtree_ref):
+            qtree_obj(&qtree_ref)
+        {
+            if (qtree_obj->root != nullptr)
+            {
+                iter_node = qtree_obj->root;
+                while (iter_node->greater != nullptr)
+                {
+                    iter_node = iter_node->greater;
+                }
+            }
+            else
+            {
+                iter_node = nullptr;
+            }
+        }
+
+        BaseReverseIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            qtree_obj(&qtree_ref),
+            iter_node(&start_node)
+        {
+        }
+
+        BaseReverseIterator(const BaseReverseIterator& orig) = default;
+        BaseReverseIterator& operator=(const BaseReverseIterator& orig) = default;
+        BaseReverseIterator(BaseReverseIterator&& orig) = default;
+        BaseReverseIterator& operator=(BaseReverseIterator&& orig) = default;
+
+        virtual ~BaseReverseIterator()
+        {
+        }
+
+        virtual T* next() = 0;
+
+        virtual bool has_next() const
+        {
+            return iter_node != nullptr;
+        }
+
+        virtual size_t get_size() const
+        {
+            return qtree_obj->size;
+        }
+
+      protected:
+        virtual Node* next_node()
+        {
+            Node* ret_node = iter_node;
+
+            if (ret_node != nullptr)
+            {
+                if (iter_node->less != nullptr)
+                {
+                    iter_node = iter_node->less;
+                    while (iter_node->greater != nullptr)
+                    {
+                        iter_node = iter_node->greater;
+                    }
+                }
+                else
+                {
+                    while (iter_node->parent != nullptr && iter_node->parent->less == iter_node)
+                    {
+                        iter_node = iter_node->parent;
+                    }
+                    iter_node = iter_node->parent;
+                }
+            }
+
+            return ret_node;
+        }
+      private:
+        const QTree* qtree_obj;
         Node*  iter_node {nullptr};
     };
 
@@ -205,6 +352,11 @@ class QTree : public dsaext::Map<K, V>
         {
         }
 
+        KeysIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            BaseIterator<K>::BaseIterator(qtree_ref, start_node)
+        {
+        }
+
         KeysIterator(const KeysIterator& orig) = default;
         KeysIterator& operator=(const KeysIterator& orig) = default;
         KeysIterator(KeysIterator&& orig) = default;
@@ -216,13 +368,47 @@ class QTree : public dsaext::Map<K, V>
 
         virtual K* next()
         {
-            K* iter_key {nullptr};
+            const K* iter_key {nullptr};
             Node* node = BaseIterator<K>::next_node();
             if (node != nullptr)
             {
                 iter_key = node->key;
             }
-            return iter_key;
+            return const_cast<K*> (iter_key);
+        }
+    };
+
+    class KeysReverseIterator : public BaseReverseIterator<K>
+    {
+      public:
+        KeysReverseIterator(QTree<K, V>& qtree_ref):
+            BaseReverseIterator<K>::BaseReverseIterator(qtree_ref)
+        {
+        }
+
+        KeysReverseIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            BaseReverseIterator<K>::BaseReverseIterator(qtree_ref, start_node)
+        {
+        }
+
+        KeysReverseIterator(const KeysReverseIterator& orig) = default;
+        KeysReverseIterator& operator=(const KeysReverseIterator& orig) = default;
+        KeysReverseIterator(KeysReverseIterator&& orig) = default;
+        KeysReverseIterator& operator=(KeysReverseIterator&& orig) = default;
+
+        virtual ~KeysReverseIterator()
+        {
+        }
+
+        virtual K* next()
+        {
+            const K* iter_key {nullptr};
+            Node* node = BaseReverseIterator<K>::next_node();
+            if (node != nullptr)
+            {
+                iter_key = node->key;
+            }
+            return const_cast<K*> (iter_key);
         }
     };
 
@@ -234,6 +420,11 @@ class QTree : public dsaext::Map<K, V>
         {
         }
 
+        ValuesIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            BaseIterator<V>::BaseIterator(qtree_ref, start_node)
+        {
+        }
+
         ValuesIterator(const ValuesIterator& orig) = default;
         ValuesIterator& operator=(const ValuesIterator& orig) = default;
         ValuesIterator(ValuesIterator&& orig) = default;
@@ -245,13 +436,47 @@ class QTree : public dsaext::Map<K, V>
 
         virtual V* next()
         {
-            V* iter_value {nullptr};
+            const V* iter_value {nullptr};
             Node* node = BaseIterator<V>::next_node();
             if (node != nullptr)
             {
                 iter_value = node->value;
             }
-            return iter_value;
+            return const_cast<V*> (iter_value);
+        }
+    };
+
+    class ValuesReverseIterator : public BaseReverseIterator<V>
+    {
+      public:
+        ValuesReverseIterator(QTree<K, V>& qtree_ref):
+            BaseReverseIterator<V>::BaseReverseIterator(qtree_ref)
+        {
+        }
+
+        ValuesReverseIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            BaseReverseIterator<V>::BaseReverseIterator(qtree_ref, start_node)
+        {
+        }
+
+        ValuesReverseIterator(const ValuesReverseIterator& orig) = default;
+        ValuesReverseIterator& operator=(const ValuesReverseIterator& orig) = default;
+        ValuesReverseIterator(ValuesReverseIterator&& orig) = default;
+        ValuesReverseIterator& operator=(ValuesReverseIterator&& orig) = default;
+
+        virtual ~ValuesReverseIterator()
+        {
+        }
+
+        virtual V* next()
+        {
+            const V* iter_value {nullptr};
+            Node* node = BaseReverseIterator<V>::next_node();
+            if (node != nullptr)
+            {
+                iter_value = node->value;
+            }
+            return const_cast<V*> (iter_value);
         }
     };
 
@@ -263,6 +488,11 @@ class QTree : public dsaext::Map<K, V>
         {
         }
 
+        NodesIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            BaseIterator<Node>::BaseIterator(qtree_ref, start_node)
+        {
+        }
+
         NodesIterator(const NodesIterator& orig) = default;
         NodesIterator& operator=(const NodesIterator& orig) = default;
         NodesIterator(NodesIterator&& orig) = default;
@@ -278,8 +508,36 @@ class QTree : public dsaext::Map<K, V>
         }
     };
 
+    class NodesReverseIterator : public BaseReverseIterator<Node>
+    {
+      public:
+        NodesReverseIterator(QTree<K, V>& qtree_ref):
+            BaseReverseIterator<Node>::BaseReverseIterator(qtree_ref)
+        {
+        }
+
+        NodesReverseIterator(QTree<K, V>& qtree_ref, Node& start_node):
+            BaseReverseIterator<Node>::BaseReverseIterator(qtree_ref, start_node)
+        {
+        }
+
+        NodesReverseIterator(const NodesReverseIterator& orig) = default;
+        NodesReverseIterator& operator=(const NodesReverseIterator& orig) = default;
+        NodesReverseIterator(NodesReverseIterator&& orig) = default;
+        NodesReverseIterator& operator=(NodesReverseIterator&& orig) = default;
+
+        virtual ~NodesReverseIterator()
+        {
+        }
+
+        virtual Node* next()
+        {
+            return BaseReverseIterator<Node>::next_node();
+        }
+    };
+
   public:
-    QTree(compare_func compare_fn):
+    QTree(const compare_func compare_fn):
         compare(compare_fn)
     {
     }
@@ -296,19 +554,19 @@ class QTree : public dsaext::Map<K, V>
 
     virtual V* get(const K* key) const
     {
-        V* value {nullptr};
-        Node* node = find_node(key);
+        const V* value {nullptr};
+        const Node* const node = find_node(key);
         if (node != nullptr)
         {
             value = node->value;
         }
-        return value;
+        return const_cast<V*> (value);
     }
 
     virtual typename dsaext::Map<K, V>::entry get_entry(const K* key) const
     {
         typename dsaext::Map<K, V>::entry qtree_entry {nullptr, nullptr};
-        Node* node = find_node(key);
+        const Node* const node = find_node(key);
         if (node != nullptr)
         {
             qtree_entry.key = node->key;
@@ -322,23 +580,402 @@ class QTree : public dsaext::Map<K, V>
         return find_node(key);
     }
 
-    // @throws std::bad_alloc, dsaext::DuplicateInsertionException
-    virtual void insert(K* key, V* value)
+    virtual K* get_first_key() const
     {
-        Node* ins_node = new Node(key, value);
-        if (!insert_node_impl(ins_node))
+        const K* key {nullptr};
+        const Node* const node = find_first_node();
+        if (node != nullptr)
         {
-            delete ins_node;
-            throw dsaext::DuplicateInsertException();
+            key = node->key;
         }
+        return const_cast<K*> (key);
+    }
+
+    virtual V* get_first_value() const
+    {
+        const V* value {nullptr};
+        const Node* const node = find_first_node();
+        if (node != nullptr)
+        {
+            value = node->value;
+        }
+        return const_cast<V*> (value);
+    }
+
+    virtual typename dsaext::Map<K, V>::entry get_first_entry() const
+    {
+        typename dsaext::Map<K, V>::entry qtree_entry {nullptr, nullptr};
+        const Node* const node = find_first_node();
+        if (node != nullptr)
+        {
+            qtree_entry.key = node->key;
+            qtree_entry.value = node->value;
+        }
+        return qtree_entry;
+    }
+
+    virtual Node* get_first_node() const
+    {
+        return find_first_node();
+    }
+
+    virtual K* get_last_key() const
+    {
+        const K* key {nullptr};
+        const Node* const node = find_last_node();
+        if (node != nullptr)
+        {
+            key = node->key;
+        }
+        return const_cast<K*> (key);
+    }
+
+    virtual V* get_last_value() const
+    {
+        const V* value {nullptr};
+        const Node* const node = find_last_node();
+        if (node != nullptr)
+        {
+            value = node->value;
+        }
+        return const_cast<V*> (value);
+    }
+
+    virtual typename dsaext::Map<K, V>::entry get_last_entry() const
+    {
+        typename dsaext::Map<K, V>::entry qtree_entry {nullptr, nullptr};
+        const Node* const node = find_last_node();
+        if (node != nullptr)
+        {
+            qtree_entry.key = node->key;
+            qtree_entry.value = node->value;
+        }
+        return qtree_entry;
+    }
+
+    virtual Node* get_last_node() const
+    {
+        return find_last_node();
+    }
+
+    virtual K* get_ceiling_key(const K* key) const
+    {
+        const K* ceiling_key {nullptr};
+        const Node* const node = find_ceiling_node_impl(key);
+        if (node != nullptr)
+        {
+            ceiling_key = node->key;
+        }
+        return const_cast<K*> (ceiling_key);
+    }
+
+    virtual V* get_ceiling_value(const K* key) const
+    {
+        const V* ceiling_value {nullptr};
+        const Node* const node = find_ceiling_node_impl(key);
+        if (node != nullptr)
+        {
+            ceiling_value = node->value;
+        }
+        return const_cast<V*> (ceiling_value);
+    }
+
+    virtual typename dsaext::Map<K, V>::entry get_ceiling_entry(const K* key) const
+    {
+        typename dsaext::Map<K, V>::entry ceiling_entry {nullptr, nullptr};
+        const Node* const node = find_ceiling_node_impl(key);
+        if (node != nullptr)
+        {
+            ceiling_entry.key = node->key;
+            ceiling_entry.value = node->value;
+        }
+        return ceiling_entry;
+    }
+
+    virtual Node* get_ceiling_node(const K* key) const
+    {
+        return find_ceiling_node_impl(key);
+    }
+
+    virtual K* get_floor_key(const K* key) const
+    {
+        const K* floor_key {nullptr};
+        const Node* const node = find_floor_node_impl(key);
+        if (node != nullptr)
+        {
+            floor_key = node->key;
+        }
+        return const_cast<K*> (floor_key);
+    }
+
+    virtual V* get_floor_value(const K* key) const
+    {
+        const V* floor_value {nullptr};
+        const Node* const node = find_floor_node_impl(key);
+        if (node != nullptr)
+        {
+            floor_value = node->value;
+        }
+        return const_cast<V*> (floor_value);
+    }
+
+    virtual typename dsaext::Map<K, V>::entry get_floor_entry(const K* key) const
+    {
+        typename dsaext::Map<K, V>::entry floor_entry {nullptr, nullptr};
+        const Node* const node = find_floor_node_impl(key);
+        if (node != nullptr)
+        {
+            floor_entry.key = node->key;
+            floor_entry.value = node->value;
+        }
+        return floor_entry;
+    }
+
+    virtual Node* get_floor_node(const K* key) const
+    {
+        return find_floor_node_impl(key);
+    }
+
+    virtual K* get_greater_key(const K* key) const
+    {
+        const K* greater_key {nullptr};
+        const Node* const node = find_greater_node_impl(key);
+        if (node != nullptr)
+        {
+            greater_key = node->key;
+        }
+        return const_cast<K*> (greater_key);
+    }
+
+    virtual V* get_greater_value(const K* key) const
+    {
+        const V* greater_value {nullptr};
+        const Node* const node = find_greater_node_impl(key);
+        if (node != nullptr)
+        {
+            greater_value = node->value;
+        }
+        return const_cast<V*> (greater_value);
+    }
+
+    virtual typename dsaext::Map<K, V>::entry get_greater_entry(const K* key) const
+    {
+        typename dsaext::Map<K, V>::entry greater_entry {nullptr, nullptr};
+        const Node* const node = find_greater_node_impl(key);
+        if (node != nullptr)
+        {
+            greater_entry.key = node->key;
+            greater_entry.value = node->value;
+        }
+        return greater_entry;
+    }
+
+    virtual Node* get_greater_node(const K* key) const
+    {
+        return find_greater_node_impl(key);
+    }
+
+    virtual K* get_less_key(const K* key) const
+    {
+        const K* less_key {nullptr};
+        const Node* const node = find_less_node_impl(key);
+        if (node != nullptr)
+        {
+            less_key = node->key;
+        }
+        return const_cast<K*> (less_key);
+    }
+
+    virtual V* get_less_value(const K* key) const
+    {
+        const V* less_value {nullptr};
+        const Node* const node = find_less_node_impl(key);
+        if (node != nullptr)
+        {
+            less_value = node->value;
+        }
+        return const_cast<V*> (less_value);
+    }
+
+    virtual typename dsaext::Map<K, V>::entry get_less_entry(const K* key) const
+    {
+        typename dsaext::Map<K, V>::entry less_entry {nullptr, nullptr};
+        const Node* const node = find_less_node_impl(key);
+        if (node != nullptr)
+        {
+            less_entry.key = node->key;
+            less_entry.value = node->value;
+        }
+        return less_entry;
+    }
+
+    virtual Node* get_less_node(const K* key) const
+    {
+        return find_less_node_impl(key);
+    }
+
+    // @throws std::bad_alloc, dsaext::DuplicateInsertionException
+    virtual void insert(const K* key, const V* value)
+    {
+        if (root == nullptr)
+        {
+            insert_node_impl(&root, nullptr, key, value);
+        }
+        else
+        {
+            Node* parent_node = root;
+            while (true)
+            {
+                const int cmp_rc = compare(key, parent_node->key);
+                if (cmp_rc < 0)
+                {
+                    if (parent_node->less == nullptr)
+                    {
+                        insert_node_impl(&parent_node->less, parent_node, key, value);
+                        break;
+                    }
+                    else
+                    {
+                        parent_node = parent_node->less;
+                    }
+                }
+                else
+                if (cmp_rc > 0)
+                {
+                    if (parent_node->greater == nullptr)
+                    {
+                        insert_node_impl(&parent_node->greater, parent_node, key, value);
+                        break;
+                    }
+                    else
+                    {
+                        parent_node = parent_node->greater;
+                    }
+                }
+                else
+                {
+                    throw dsaext::DuplicateInsertException();
+                }
+            }
+        }
+    }
+
+    // @throws std::bad_alloc
+    virtual V* insert_or_update(const K* key, const V* value)
+    {
+        const V* prev_value {nullptr};
+
+        if (root == nullptr)
+        {
+            insert_node_impl(&root, nullptr, key, value);
+        }
+        else
+        {
+            Node* parent_node = root;
+            while (true)
+            {
+                const int cmp_rc = compare(key, parent_node->key);
+                if (cmp_rc < 0)
+                {
+                    if (parent_node->less == nullptr)
+                    {
+                        insert_node_impl(&parent_node->less, parent_node, key, value);
+                        break;
+                    }
+                    else
+                    {
+                        parent_node = parent_node->less;
+                    }
+                }
+                else
+                if (cmp_rc > 0)
+                {
+                    if (parent_node->greater == nullptr)
+                    {
+                        insert_node_impl(&parent_node->greater, parent_node, key, value);
+                        break;
+                    }
+                    else
+                    {
+                        parent_node = parent_node->greater;
+                    }
+                }
+                else
+                {
+                    prev_value = parent_node->value;
+                    parent_node->value = value;
+                    break;
+                }
+            }
+        }
+        return const_cast<V*> (prev_value);
+    }
+
+    // @throws std::bad_alloc
+    virtual V* update(const K* key, const V* value)
+    {
+        const V* prev_value {nullptr};
+
+        Node* node = find_node(key);
+        if (node != nullptr)
+        {
+            prev_value = node->value;
+            node->value = value;
+        }
+
+        return const_cast<V*> (prev_value);
     }
 
     // @throws std::bad_alloc, dsaext::DuplicateInsertionException
     virtual void insert_node(Node* node)
     {
-        if (!insert_node_impl(node))
+        if (root == nullptr)
         {
-            throw dsaext::DuplicateInsertException();
+            root = node;
+            ++size;
+        }
+        else
+        {
+            Node* parent_node = root;
+            while (true)
+            {
+                int cmp_rc = compare(node->key, parent_node->key);
+                if (cmp_rc < 0)
+                {
+                    if (parent_node->less == nullptr)
+                    {
+                        parent_node->less = node;
+                        node->parent = parent_node;
+                        ++size;
+                        rebalance_insert(node, parent_node);
+                        break;
+                    }
+                    else
+                    {
+                        parent_node = parent_node->less;
+                    }
+                }
+                else
+                if (cmp_rc > 0)
+                {
+                    if (parent_node->greater == nullptr)
+                    {
+                        parent_node->greater = node;
+                        node->parent = parent_node;
+                        ++size;
+                        rebalance_insert(node, parent_node);
+                        break;
+                    }
+                    else
+                    {
+                        parent_node = parent_node->greater;
+                    }
+                }
+                else
+                {
+                    throw dsaext::DuplicateInsertException();
+                }
+            }
         }
     }
 
@@ -432,61 +1069,188 @@ class QTree : public dsaext::Map<K, V>
         return node;
     }
 
-    inline bool insert_node_impl(Node* ins_node)
+    inline Node* find_first_node() const
     {
-        bool inserted = true;
-
-        if (root == nullptr)
+        Node* node = root;
+        if (node != nullptr)
         {
-            root = ins_node;
-            ++size;
-        }
-        else
-        {
-            Node* parent_node = root;
-            while (true)
+            while (node->less != nullptr)
             {
-                int cmp_rc = compare(ins_node->key, parent_node->key);
-                if (cmp_rc < 0)
+                node = node->less;
+            }
+        }
+        return node;
+    }
+
+    inline Node* find_last_node() const
+    {
+        Node* node = root;
+        if (node != nullptr)
+        {
+            while (node->greater != nullptr)
+            {
+                node = node->greater;
+            }
+        }
+        return node;
+    }
+
+    inline Node* find_ceiling_node_impl(const K* key) const
+    {
+        Node* node = root;
+        while (node != nullptr)
+        {
+            int cmp_rc = compare(key, node->key);
+            if (cmp_rc < 0)
+            {
+                if (node->less != nullptr)
                 {
-                    if (parent_node->less == nullptr)
-                    {
-                        parent_node->less = ins_node;
-                        ins_node->parent  = parent_node;
-                        ++size;
-                        rebalance_insert(ins_node, parent_node);
-                        break;
-                    }
-                    else
-                    {
-                        parent_node = parent_node->less;
-                    }
-                }
-                else
-                if (cmp_rc > 0)
-                {
-                    if (parent_node->greater == nullptr)
-                    {
-                        parent_node->greater = ins_node;
-                        ins_node->parent     = parent_node;
-                        ++size;
-                        rebalance_insert(ins_node, parent_node);
-                        break;
-                    }
-                    else
-                    {
-                        parent_node = parent_node->greater;
-                    }
+                    node = node->less;
                 }
                 else
                 {
-                    inserted = false;
+                    break;
+                }
+            }
+            else
+            if (cmp_rc > 0)
+            {
+                if (node->greater != nullptr)
+                {
+                    node = node->greater;
+                }
+                else
+                {
+                    while (node->parent != nullptr && node->parent->greater == node)
+                    {
+                        node = node->parent;
+                    }
+                    node = node->parent;
+                    break;
+                }
+            }
+            else
+            {
+                break;
+            }
+        }
+        return node;
+    }
+
+    inline Node* find_floor_node_impl(const K* key) const
+    {
+        Node* node = root;
+        while (node != nullptr)
+        {
+            int cmp_rc = compare(key, node->key);
+            if (cmp_rc < 0)
+            {
+                if (node->less != nullptr)
+                {
+                    node = node->less;
+                }
+                else
+                {
+                    while (node->parent != nullptr && node->parent->less == node)
+                    {
+                        node = node->parent;
+                    }
+                    node = node->parent;
+                    break;
+                }
+            }
+            else
+            if (cmp_rc > 0)
+            {
+                if (node->greater != nullptr)
+                {
+                    node = node->greater;
+                }
+                else
+                {
+                    break;
+                }
+            }
+            else
+            {
+                break;
+            }
+        }
+        return node;
+    }
+
+    inline Node* find_greater_node_impl(const K* key) const
+    {
+        Node* node = root;
+        while (node != nullptr)
+        {
+            int cmp_rc = compare(key, node->key);
+            if (cmp_rc < 0)
+            {
+                if (node->less != nullptr)
+                {
+                    node = node->less;
+                }
+                else
+                {
+                    break;
+                }
+            }
+            else
+            {
+                if (node->greater != nullptr)
+                {
+                    node = node->greater;
+                }
+                else
+                {
+                    while (node->parent != nullptr && node->parent->greater == node)
+                    {
+                        node = node->parent;
+                    }
+                    node = node->parent;
                     break;
                 }
             }
         }
+        return node;
+    }
 
-        return inserted;
+    inline Node* find_less_node_impl(const K* key) const
+    {
+        Node* node = root;
+        while (node != nullptr)
+        {
+            int cmp_rc = compare(key, node->key);
+            if (cmp_rc > 0)
+            {
+                if (node->greater != nullptr)
+                {
+                    node = node->greater;
+                }
+                else
+                {
+                    break;
+                }
+            }
+            else
+            {
+                if (node->less != nullptr)
+                {
+                    node = node->less;
+                }
+                else
+                {
+                    while (node->parent != nullptr && node->parent->less == node)
+                    {
+                        node = node->parent;
+                    }
+                    node = node->parent;
+                    break;
+                }
+            }
+        }
+        return node;
     }
 
     inline void remove_node_impl(Node* rm_node)
@@ -495,6 +1259,24 @@ class QTree : public dsaext::Map<K, V>
         delete rm_node;
     }
 
+    // @throws std::bad_alloc
+    inline void insert_node_impl(
+        Node** const ref_ins_node,
+        Node* const parent_node,
+        const K* const key,
+        const V* const value
+    )
+    {
+        Node* ins_node = new Node(key, value);
+        *ref_ins_node = ins_node;
+        ins_node->parent = parent_node;
+        ++size;
+        if (parent_node != nullptr)
+        {
+            rebalance_insert(ins_node, parent_node);
+        }
+    }
+
     inline void unlink_node_impl(Node* rm_node)
     {
         --size;
@@ -542,20 +1324,18 @@ class QTree : public dsaext::Map<K, V>
             // find replacement node
             if (rm_node->balance == -1)
             {
-                for (replace_node = rm_node->less;
-                     replace_node->greater != nullptr;
-                     replace_node = replace_node->greater)
+                replace_node = rm_node->less;
+                while (replace_node->greater != nullptr)
                 {
-                    // intentional no-op block
+                    replace_node = replace_node->greater;
                 }
             }
             else
             {
-                for (replace_node = rm_node->greater;
-                     replace_node->less != nullptr;
-                     replace_node = replace_node->less)
+                replace_node = rm_node->greater;
+                while (replace_node->less != nullptr)
                 {
-                    // intentional no-op block
+                    replace_node = replace_node->less;
                 }
             }
             Node* rot_node = replace_node->parent;
@@ -1114,7 +1894,7 @@ class QTree : public dsaext::Map<K, V>
     Node*  root {nullptr};
     size_t size {0};
 
-    compare_func compare;
+    const compare_func compare;
 };
 
 #endif	/* QTREE_H */
